import { NextFunction, Request, Response, Router } from "express";
import { BaseRoute } from "./route";
import * as redis from "redis";
import { RedisClient } from "redis";

/**
 * / route
 *
 * @class User
 */
export class StatsRoute extends BaseRoute {

  public static client: RedisClient = redis.createClient();

  /**
   * Create the routes.
   *
   * @class StatsRoute
   * @method create
   * @static
   */
  public static create(router: Router) {
    //log
    console.log("[StatsRoute::create] Creating stats route.");

    //add stats route
    router.get("/stats", (req: Request, res: Response, next: NextFunction) => {
      new StatsRoute().stats(req, res, next);
    });
    router.get("/stats/delete", (req: Request, res: Response, next: NextFunction) => {
      new StatsRoute().statsDelete(req, res, next);
    });
    router.get("/stats/data", (req: Request, res: Response, next: NextFunction) => {
      new StatsRoute().statsData(req, res, next);
    });
  }

  /**
   * Constructor
   *
   * @class StatsRoute
   * @constructor
   */
  constructor() {
    super();
  }

  /**
   * The stats route.
   *
   * @class StatsRoute
   * @method stats
   * @param req {Request} The express Request object.
   * @param res {Response} The express Response object.
   * @next {NextFunction} Execute the next method.
   */
  public stats(req: Request, res: Response, next: NextFunction) {
    var jsonStats = {
      title: 'Statistike',
      vsebujeIme: '0',
      vsebujePriimek: '0',
      vsebujeUlico: '0',
      vsebujeObliko: '0',
      vsebujeDavcno: '0',
      vsebujeMaticno: '0',
      vsebujeFilter: '0',
      obdelanih: '0',
      niziBrezOP: '0'
    };
    var vse = 9;
    var koncano = 0;

    StatsRoute.client.get('stats:vsebujeIme', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeIme = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:vsebujePriimek', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujePriimek = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:vsebujeUlico', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeUlico = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:vsebujeObliko', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeObliko = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:vsebujeDavcno', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeDavcno = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:vsebujeMaticno', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeMaticno = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:vsebujeFilter', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeFilter = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:obdelanih', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.obdelanih = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:niziBrezOP', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.niziBrezOP = result;
        koncano++;
        if(vse == koncano) {
          res.render('stats', jsonStats);
        }
      }
    });
  }

  /**
   * The stats/delete route.
   *
   * @class StatsRoute
   * @method statsDelete
   * @param req {Request} The express Request object.
   * @param res {Response} The express Response object.
   * @next {NextFunction} Execute the next method.
   */
  public statsDelete(req: Request, res: Response, next: NextFunction) {
    var vse = 9;
    var koncano = 0;
  
    StatsRoute.client.set('stats:vsebujeIme', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });

    StatsRoute.client.set('stats:vsebujePriimek', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });
  
    StatsRoute.client.set('stats:vsebujeUlico', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });
  
    StatsRoute.client.set('stats:vsebujeObliko', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });
  
    StatsRoute.client.set('stats:vsebujeDavcno', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });

    StatsRoute.client.set('stats:vsebujeMaticno', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });
  
    StatsRoute.client.set('stats:vsebujeFilter', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });
  
    StatsRoute.client.set('stats:obdelanih', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });

    StatsRoute.client.set('stats:niziBrezOP', '0', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: err, stack: err} })
      } else {
        koncano++;
        if(vse == koncano) {
          res.redirect('/stats');
        }
      }
    });
  }

  /**
   * The stats/data route.
   *
   * @class StatsRoute
   * @method statsData
   * @param req {Request} The express Request object.
   * @param res {Response} The express Response object.
   * @next {NextFunction} Execute the next method.
   */
  public statsData(req: Request, res: Response, next: NextFunction) {
    var jsonStats = {
      title: 'Statistike',
      vsebujeIme: '0',
      vsebujePriimek: '0',
      vsebujeUlico: '0',
      vsebujeObliko: '0',
      vsebujeDavcno: '0',
      vsebujeMaticno: '0',
      vsebujeFilter: '0',
      obdelanih: '0',
      niziBrezOP: '0'
    };
    var vse = 9;
    var koncano = 0;
  
    StatsRoute.client.get('stats:vsebujeIme', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeIme = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:vsebujePriimek', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujePriimek = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:vsebujeUlico', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeUlico = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:vsebujeObliko', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeObliko = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:vsebujeDavcno', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeDavcno = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:vsebujeMaticno', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeMaticno = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:vsebujeFilter', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.vsebujeFilter = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });
  
    StatsRoute.client.get('stats:obdelanih', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.obdelanih = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });

    StatsRoute.client.get('stats:niziBrezOP', function(err, result) {
      if(err) {
        res.render('error', { messge: err.message, error: { status: 0, stack: 0} })
      } else {
        jsonStats.niziBrezOP = result;
        koncano++;
        if(vse == koncano) {
          res.send(jsonStats);
        }
      }
    });
  }
}